# Privacy Notice for Hubs and Spoke

Version 3.3, November 15, 2019

## At Mozilla (that’s us), we believe that privacy is fundamental to a healthy internet.

In this Privacy Notice, we explain what data may be accessible to Mozilla or others when you use [Hubs](https://hubs.mozilla.com) or [Spoke](https://hubs.mozilla.com/spoke). We also adhere to the practices outlined in the Mozilla [privacy policy](https://www.mozilla.org/en-US/privacy/) for how we receive, handle, and share information we collect from Hubs.

## Things you should know:

<details open>
  <summary>
    <strong>Your presence and communications are sent to Mozilla and other Room participants.</strong>
  </summary>

- **Avatar data**: We receive and send to others in the Room the name and likeness of your Avatar, its position in the Room, and your interactions with objects in the Room. 
  - **Custom Avatars**: You may create a custom avatar. If you create an account, Mozilla will store your custom avatar, associated with your account.
  - **Stock Avatars**: If you choose one of the regular avatars, you may login to store information about your avatar with Mozilla. You can optionally store information about your Avatar in your browser’s local storage if you do not log in to your account.  
- **Room data**: Rooms are publicly accessible to anyone with the URL. Mozilla receives data about the virtual objects and Avatars in a Room and shares that data with others in the Room.     
- **Voice data**: If your microphone is on, Mozilla receives and sends audio to other users in the Room. Mozilla does not record or store the audio.  *Be aware that once you agree to let Hubs use your microphone, it will stay on as long as you remain in a Hubs room, unless you turn it off.*
- **In-Room Photos and Video**: You can use the camera tool to take photos and video unless the room owner has disabled this feature. The camera tool displays a red icon during filming. Your avatar will also indicate to others with a red icon that you are filming and capturing audio from enabled microphones in the room.
  - Data retention: If you take a photo or video, the file is stored locally on your device and uploaded to Mozilla’s servers so that it can be shared within the room. Pinned files are stored permanently in your room unless you remove it. Unpinned files are deleted from Mozilla’s servers after 72 hours. 
- **Third-Party Communications Services**: Hubs allows you to connect some third-party services to a Hubs room.
  - **Twitter**: Logging into your Twitter account from Hubs allows you to tweet and share 2D images. Any content you tweet will be shared with Twitter and published publicly on the Twitter platform. You can see Twitter’s [Privacy Policy](https://twitter.com/en/privacy) for more information.
  - **Discord**: Hubs allows you to connect Discord to a Hubs room. When someone has connected Discord to a Hubs room, we store access tokens and the server and channel IDs that have been connected. We will synchronize chat messages, room changes, 2D and 3D objects you create, and whether a user joins or leaves with the connected Discord channel, but do not log any of these synchronized messages. For diagnostics, we log aggregated counts such as the number of messages and users who have joined relevant channels. You can see [Discord’s Privacy Policy](https://discordapp.com/privacy) for more information.

- You can learn more by looking at the [code itself](https://github.com/mozilla/hubs) for Hubs. [Janus SFU](https://github.com/mozilla/janus-plugin-sfu), [Reticulum](https://github.com/mozilla/reticulum), [Hubs](https://github.com/mozilla/hubs), [Hubs-Ops](https://github.com/mozilla/hubs-ops), [Discord Bot](https://github.com/MozillaReality/hubs-discord-bot)

</details>

<details open>
  <summary>
    <strong>Mozilla receives data you create and share with Spoke and Hubs.</strong>
  </summary>

- **Images, Video, and Objects**: Mozilla receives video and image file links to process and display them in the Hubs Room. Mozilla stores this data as long as you remain in the Room. If you pin a video, image, or object in the Room, Mozilla will store that information until the object or room is deleted.
- **Scenes and Avatars You Share**: Mozilla receives 3D Room model links, 3D Avatar information, and the name of the Room or Avatar in order to process and display the Room or Avatar. Mozilla stores the name and the URL for the link you share so you and others with the link to the Room can use it again. 
- **Search**: You can search for images, GIFs, and 3D Models to share in Hubs. We use third-party search providers. When you search, we will send your searches to these third parties, which have their own privacy policies. 
  - Microsoft Bing Video & Image Search: https://privacy.microsoft.com/en-us/privacystatement
  - Tenor: https://tenor.com/legal-privacy
  - Sketchfab: https://sketchfab.com/privacy
  - Google Poly & YouTube: https://policies.google.com/privacy
  - Twitch: https://www.twitch.tv/p/legal/privacy-policy/

<details open>
  <summary>
    <strong>Mozilla receives data when you create and publish Scenes with Spoke or create a custom Avatar.</strong>
  </summary>

- **Scenes and Avatars You Create**: When you create a scene with Spoke or a custom avatar, Mozilla receives a copy of that scene or avatar. Mozilla stores that data in order to be able to process and display the scene or avatar through Hubs. 
- **Publishing Your Scene or Avatar**: When you publish a scene or avatar to Hubs, Mozilla will ask for your email address to send you a link to verify your email. Mozilla will receive and store your email address to allow you to log in and view your 3D Room models and Avatars. Mozilla stores a hashed version of email addresses you use to publish a scene, so the stored versions are not available in readable form. 
- **Remixing, Promotion, and Favorites**: When you publish a scene or avatar to Hubs, you have the option to “Allow Remixing with Creative Commons [CC-BY 3.0](https://creativecommons.org/licenses/by/3.0/)” or allow Mozilla to promote your scene or avatar. If you choose one or both of these options, Mozilla will share your scene or avatar publicly and you will have the option of including attribution information, which will also be publicly available. If you mark a Room or Avatar as a Favorite, Mozilla will store that information associated with your account.

- You can learn more by looking at the [code itself](https://github.com/mozilla/spoke) for Spoke. 
</details>

<details open>
  <summary>
    <strong>Mozilla receives technical and interaction data to improve performance and stability for Hubs.</strong>
  </summary>

- **Account data**: You do not need to create an account to use Hubs. If you decide to create an account with your email address, we receive your email address to send you a login link, but stores only a hashed version of your email address. If you decide to use Discord to create your account, we receive the email address associated with your Discord account and your Discord avatar. 
- **Technical data**: We receive and store data about Room URLs and names; the type of device you use to interact with Hubs, as well as its operating system, language, the name and version of browser; and other data to load and operate the Room.
- **Interaction data**: We receive data about your interactions with the Hubs service itself such as the number of Rooms created, messages sent through or to third-party services like Slack and Discord, the maximum number of users in a particular room at one same time, the start and end time of a user’s interaction with Hubs, the amount of time a user interacts with Hubs through Virtual Reality, the first time in a particular month or day that a user begins to use Hubs. Mozilla uses third party services to store and analyze these operational messages. We also also use Google Analytics, which places a cookie on your device, to obtain metrics on how users engage with our websites. This helps us to improve site content.
- **Website data**: We use Google Analytics to better understand how you interact with Hubs apps and websites. For example, we may use cookies to track de-identified information such as the number of Hubs rooms you create or enter, interactions with buttons and menus, and session length. To consistently improve the Hubs experience, we collect information about browser type and version, viewport size, and screen resolution. This allows us to understand how people interact with Hubs so we can improve it. We also collect your location, and the language setting in your browser to make sure it looks right for you. If you’d prefer to opt out of Google Analytics data collection, you can install the [Google Analytics Opt-out Browser Add-on](https://tools.google.com/dlpage/gaoptout), which stops data transmission to Google Analytics.
- **Error Data**: In order to diagnose problems, Hubs sends Mozilla logs of error messages (which include the Room URL, response time for requests, the page you were on when you encountered the error, your operating system, browser information, and may include your IP address). 
- You can learn more by looking at the [code itself](https://github.com/mozilla/hubs) for Hubs.  [Janus SFU](https://github.com/mozilla/janus-plugin-sfu), [Reticulum](https://github.com/mozilla/reticulum), [Hubs](https://github.com/mozilla/hubs), [Hubs-Ops](https://github.com/mozilla/hubs-ops), [Discord Bot](https://github.com/MozillaReality/hubs-discord-bot)
</details>
